# JavaScript Coding Styleguide

*This guide contains a list of recommendations and patterns we use in our code to create clean, consistent, easy-to-read & easy-to-extend code.
Discussion and update is always welcome, so do not hesitate to reach out.*

*This style guide focuses on pure JavaScript recommendations and patterns we use in our code. For React specific content, please see our [React Coding Styleguide](../react/README.md)*


## Table of Contents

  1. [Syntax](#syntax)
    1. [Line length](#line-length)
    1. [Line breaks](#line-length)
    1. [Indentation, Quotes and Semicolons](#Indentation,-Quotes-and-Semicolons)
    1. [Parentheses and brackets](#parentheses-and-brackets)
    1. [Declarations](#declarations)
    1. [Statements](#statements)
    1. [Naming](#naming)
    1. [Logical operators](#logical-operators)
    1. [Ternary operators](#ternary-operators)
    1. [Functions](#functions)
    1. [String operators](#string-operators)
  1. [Documentation & comments](#documentation-&-comments)
    1. [Docs](#docs)
    1. [Comments](#comments)
  1. [Language specific use](#language-specific-use)

## Syntax

### Line length
- 120 characters per line.
  > Why? 120 characters is easier to read and it enables to use split screens without the need of horizontal scrolling.

### Line breaks
- Use line breaks to increase the readability of the code; especially before returns.

```javascript
// bad
const a = 'bar';
const foo = () => {
  // this is calculating something
  const something = . . .;
  // this is calculating something else
  const else = . . .;
  return { left: something, right: else };
}

// good
const a = 'bar';

const foo = () => {
  // this is calculating something
  const something = . . .;

  // this is calculating something else
  const else = . . .;

  return {
    left: something,
    right: else
  };
}
```

### Indentation, Quotes and Semicolons
- Use two space indentation and NEVER EVER mix tabs and spaces.

- Use single quotes (`'`)

```javascript
// bad
const a = "bar";

// good
const a = 'bar';
```

- Use semicolons, for detailed explanation see [JavaScript Semicolon Insertion](http://inimino.org/~inimino/blog/javascript_semicolons).

### Parenthesis and brackets

- Open the brackets on the same line as is the statement.

```javascript
// bad
if (something)
{
  . . .
}

// good
if (something) {
  . . .
}
```

- Do not use space inside round brackets.

```javascript
// bad
const elementHeight = windowHeight * ( elementHeight + spacer );

if ( something ) {
  . . .
}

// good
const elementHeight = windowHeight * (elementHeight + spacer);

if (something) {
  . . .
}
```

- Use space inside curly brackets.

```javascript
// bad
const position = ({x, y}) => {a: x, y:b};

// good
const position = ({ x, y }) => { a: x, y: b };
```

- Arrays:

    Do not use spaces after opening square brackets and before closing square brackets.

    If array includes short items, put them in a one row, otherwise put one time per line.

    Do not use trailing comma.

```javascript
// bad
const colors = [
'blue', 'red', 'white'
];

const rows = [ { id: 234, color: 'blue', . . . }, { id: 456, color: 'white', . . . }, { id: 654, color: 'yellow', . . . } ];

const arrayWithTrailingComma = [
  { commaAfterwards },
  { commaAfterwards },
  { commaAfterwards },
];

// good
const colors = ['blue', 'red', 'white'];

const rows = [
  { id: 234, color: 'blue', . . . },
  { id: 456, color: 'white', . . . },
  { id: 654, color: 'yellow', . . . }
];

const arrayWithourTrailingComma = [
  { commaAfterwards },
  { commaAfterwards },
  { noCommaAfterwards }
];
```

- Objects:

    Use new lines to increate the readability.

    Use shorthand property names if possible.

    Do not use trailing comma.

```javascript
// bad
const circle = {
  stroke: stroke,
  border: border
};

const properties = {
  id: getId(),
  stroke: 'blue'
};

// good
const circle = {
  stroke,
  border
};

const properties = {
  id: getId(),
  stroke: 'blue'
};
```

### Declarations
- Use const as a default variable declaration, unless a variable needs to be reassigned.

  The var keyword and undeclared variable must not be used.

```javascript
// bad
let foo = something;
bazz = someFunc();

var bar = someFunc();
if (bar) {
  bar = someOther();
}

// good
const foo = something;
const bazz = someFunc();

let bar = someFunc();
if (bar) {
  bar = someOther();
}
```

- If conditions are non-trivial, assign them to variable and use a descriptive name.

  If there are more than three conditions, put them on a new line with the logical operator first and condition afterwards, see example in [Logical operators](#logical-operators).

```javascript
// bad
if (borderWidth !== 0 && boxHeight < minimalHeight && heading !== '') {
  . . .
}

// good
const createHeading = borderWidth !== 0 && boxHeight < minimalHeight && heading !== '';

if (createHeading) {
  . . .
}
```

### Naming
- Use:
    * lowerCamelCase - variables, functions, properties
    * UpperCamelCase - class names
    * UPPERCASE - static class properties

```javascript
// bad
const pi = . . .;
const user_info = . . .;
const isUserVALID = . . .;

const validate_all_users = () => { . . . };


// good
const PI = . . .;
const userInfo = . . .;
const isUserValid = . . .;

const validateAllUsers = () => { . . . };
```

- For private properties and methods use underscore (```_```) as a name prefix.

```javascript
// bad
const calculateSomethingPrivate = () => { . . . };
const createSomething = () => { . . . };

export createSomething;

// good
const _calculateSomethingPrivate = () => { . . . };
const createSomething = () => { . . . };

export createSomething;
```

- Use easy-to-understand and descriptive names.

    Do not use abbreviations (```i``` is exception, you can use ```index``` or ```i``` BUT NOT ```ix``` NOR ```indx```).

    Explicit is better than implicit. Always.

```javascript
// bad
const numberOfEls = . . .

locations.forEach(l => {
  . . .
  findName(l);
});

// good
const numberOfListItems = . . .

locations.forEach(location => {
  . . .
  findName(location);
});
```

- If the name would became too long, use shorter descriptive name and put explanation in a comment above it.

```javascript
// bad
const circlesIntersectingVisiblePageElementsList = . . .

// good
// list of all circles which are intersecting visible elements on the page
const intersectingCircles = . . .
```

### Logical operators
- When using more than one logical operator, put each operator on a new line and then include the statement.
  > Why? It's easier to understand and read the code.

```javascript
// bad
if (height > 0 &&
    i >= numberOfElements - space &&
    valid) {
  position = numberOfElements * elementHeight + space;
}


// good
if (height > 0
  && i >= numberOfElements - space
  && valid) {
  position = numberOfElements * elementHeight + space;
}
```

### Ternary operators
- Prefer ternary operators over if statements.

```javascript
// bad
let foo = somethingInvalid;

if (somethingCalled) {
    foo = somethingValid;
}

// good
const foo = somethingCalled ? somethingValid : somethingInvalid;
```

- Put the question mark and colon on a new line with indentation if the statements are complex.

```javascript
// bad
const foo = (foo || bar || bazz) ? (somethingElse + somethingA * 2) : (somethingElse - somethingA / 2);


// good
const foo = isItSomething ? somethingShort : somethingElseShort;

const foo = (typeof something === "number")
  ? (somethingNotThatShort + somethingA * 2)
  : (somethingElse - somethingA / 2);

const foo = (somethingA || somethingB || somethingC || somethingD)
  ? (
    // multiple
    // lines
    // of
    // code
  ) : '';
```

### Functions

- Use default arguments instead of conditions.

```javascript
// bad
const heading = (text, color) {
  const headingText = text || 'Default title';
  const headingColor = color || 'black';
  . . .
}

// good
const heading = (text = 'Default title', color = 'black') {
  . . .
}
```

- If a function has more than three arguments, consolidate them.
```javascript
// bad
const heading = (text, color, border, size) {
  . . .
}

// good
const heading = ({ text, color, border, size }) {
  . . .
}
```

- If methods can be chained, put one call on each line with indentation.

```javascript
// bad
svg.select('.boxes').select('.triangles')
.attr('stroke', . . . ).on('mouseclick', . . . );

// good
svg.select('.boxes')
  .select('.triangles')
  .attr('stroke', . . . )
  .on('mouseclick', . . . );
```

### String operators
- For initiating string, use single quotes,
  e.g. ```const defaultLabel = '';```

- For names, labels and such, use string with single quotes,
  e.g. ```const defaultLabel = 'Heading number one';```

- For strings which include variable, use [template literals](https://developer.mozilla.org/en/docs/Web/JavaScript/Reference/Template_literals) instead of concatenation.

```javascript
// bad
const foo = 'Version: ' + params.v;
const foo = 'Rainy days: ' + rain + ' out of ' + allDays;

// good
const foo = `Version: ${ params.v }`;
const foo = `Rainy days: ${ rain } out of ${ allDays }`;
```

## Documentation & comments

### Docs

- Write documentation, see [JSDoc](#http://usejsdoc.org/) if in doubt.

- Be precise and explicit.

- Use sentences for function descriptions and use statements for variable, parameter and argument description.

- Try to be descriptive even if the name of the variable seems to be obvious,
  e.g. `width` replace with `width of the window`.

- For classes and components / models, describe what it should do and give an example of usage.

- If an object is returned or expected to be passed as an argument, describe its content or given an example.

```javascript
// bad
/**
 * _getLayout - get the layout
 * @param  {Number} width
 * @param  {Number} height
 * @return {Object} split layout
 */
. . .

// good
/**
 * _getLayout - Get the updated main split layout.
 * @param  {Number} width to base layout off
 * @param  {Number} height of the layout
 * @return {Object} split layout positions, e.g. { splitLayout: { left: . . . }, width: . . ., height: . . . }
 */
. . .
```

- If needed use lists but always with a bullet character.
    > Why? Automatic tools which can generate documentation from source code will ignore plain text formatting.

```javascript
// bad
/**
 * The following is needed:
 *      available width
 *      available height
 *      list of things
 *      elements
 */

// good
/**
 * The following is needed:
 *  - available width
 *  - available height
 *  - list of things
 *  - elements
 */
```

### Comments

- Use single line comments if needed. Do not overuse comments, rename variable and function names instead.

- Put one line comments above the statement which it belongs to instead of after.

  If the variable has a descriptive name, avoid the comment.

```javascript
// bad
const something = somethingElse(); // this is doing something


// good

// this is doing something
const something = somethingElse();
```

## Language specific use

- Use strict equality (```===```) instead of equality (```==```).

- Do not use with or eval.

- Try to make your functions pure.
    > Why? They are much easier to test than functions with side effects, moreover, they are easier to reuse or modify.

```javascript
// bad
const addItemToCart = (cart, item) => {
  cart.push({ item, date: Date.now() });
};

// good
const addItemToCart = (cart, item) => {
  return [...cart, { item, date : Date.now() }];
};
```
- Use arrow functions.
    > Why? Avoiding .bind(this)

- (TODO: DISCUSS - this may take longer discussion
      but needs to be considered - @Ottis used assert functions in the tree module,
      so you can see how it can be used in our code.
      The question is - do we want to check types even though
      we mostly use primitive types and have a consistent API?)

  Avoid type checking.

    - please see [https://github.com/ryanmcdermott/clean-code-javascript#avoid-type-checking-part-1](https://github.com/ryanmcdermott/clean-code-javascript#avoid-type-checking-part-1)
    - please see [https://github.com/ryanmcdermott/clean-code-javascript#avoid-type-checking-part-2](https://github.com/ryanmcdermott/clean-code-javascript#avoid-type-checking-part-2)

```javascript
// bad
const combine = (val1, val2) => {
  if (typeof val1 === 'number' && typeof val2 === 'number' ||
      typeof val1 === 'string' && typeof val2 === 'string') {
    return val1 + val2;
  }

  throw new Error('Must be of type String or Number');
}

// good
const combine = (val1, val2) => {
  return val1 + val2;
}
```

- Use Promises, nobody wants callback hell.
    If possible, use [async](https://developer.mozilla.org/en-US/docs/Web/JavaScript/Reference/Statements/async_function) and [await](https://developer.mozilla.org/en-US/docs/Web/JavaScript/Reference/Operators/await).
