# React Coding Styleguide

*This guide is a list of recommendations and patterns we use in our code to create clean, consistent, easy-to-read & easy-to-extend code.*

*It is based on [Airbnb React/JSX Style Guide](https://github.com/airbnb/javascript/blob/master/react) with modifications and additions made from our existing conventions.
All general rules regarding JavaScript can be found in our [JavaScript Coding Styleguide](../javascript/README.md)*


## Table of Contents

  1. [Basic Rules](#basic-rules)
  1. [Class vs `React.createClass` vs stateless](#class-vs-reactcreateclass-vs-stateless)
  1. [Naming](#naming)
  1. [Alignment](#alignment)
  1. [Quotes](#quotes)
  1. [Spacing](#spacing)
  1. [Props](#props)
  1. [Refs](#refs)
  1. [Parentheses](#parentheses)
  1. [Tags](#tags)
  1. [Methods](#methods)
  1. [Event handlers](#event-handlers)
  1. [Ordering](#ordering)
  1. [`isMounted`](#ismounted)

## Basic Rules

- Only include one React component per file.
  - (added) If there is a stateless component and a simple stateful component they can share the same folder.
- Always use JSX syntax.

## Class vs `React.createClass` vs stateless

- Use `class Example extends React.Component` instead of `React.createClass`.
    eslint: [`react/prefer-es6-class`](https://github.com/yannickcr/eslint-plugin-react/blob/master/docs/rules/prefer-es6-class.md) [`react/prefer-stateless-function`](https://github.com/yannickcr/eslint-plugin-react/blob/master/docs/rules/prefer-stateless-function.md)

```jsx
// bad
const Listing = React.createClass({
  // . . .
  render() {
    return <div>{this.state.hello}</div>;
  }
});

// good
class Listing extends React.Component {
  // . . .
  render() {
    return <div>{this.state.hello}</div>;
  }
}
```

- Use `extends React.Component` or `extends Component` as it suits the particular class.

```jsx
// good
import { Component } from 'react';
class Listing extends Component {
    // . . .
  }
}

// good
import React from 'react';
class Listing extends React.Component {
    // . . .
  }
}
```

- Prefer arrow functions instead of normal functions or classes:

```jsx
// bad
class Listing extends React.Component {
  render() {
    return <div>{this.props.hello}</div>;
  }
}

function Listing({ hello }) {
  return <div>{hello}</div>;
}

// good
const Listing = ({ hello }) => (
  <div>{hello}</div>
);

// also possible to create as the following if it fits on one line
const Listing = ({ hello }) => <div>{hello}</div>;
```

## Naming

- **Extensions**: Use `.jsx` extension for React components.

- **Filename**:
  - (modified) Use kebab-case (all lowercase with `-` separating words) for filenames and folder names. E.g., `orbit-chart` or `orbit-chart.jsx` respectively.

- **Folder Structure**:
  - (added) The folder should include the JSX component with the same name as the folder. E.g. `/orbit-chart/orbit-chart.jsx`.
  - (added) Use the same name as the folder but add the extension for other files. E.g. `/orbit-chart/orbit-chart.css`.
  - (added) Use the same name as the folder but add the extension `test.js` for test files. E.g. `/orbit-chart/orbit-chart.test.js`.

- **Reference Naming**: Use PascalCase for React components and camelCase for their instances. eslint: [`react/jsx-pascal-case`](https://github.com/yannickcr/eslint-plugin-react/blob/master/docs/rules/jsx-pascal-case.md)

```jsx
// bad
import satelliteTrail from '../satellite-trail/satellite-trail';

// good
import SatelliteTrail from '../satellite-trail/satellite-trail';

// bad
const SatelliteTrail = <SatelliteTrail />;

// good
const satelliteTrail = <SatelliteTrail />;
```

- **Component Naming**: Use the filename as the component name. For example, `ReservationCard.jsx` should have a reference name of `ReservationCard`. However, for root components of a directory, use `index.jsx` as the filename and use the directory name as the component name:

```jsx
// bad
import Footer from './Footer/Footer';

// bad
import Footer from './Footer/index';

// good
import Footer from './Footer';
```

- **Higher-order Component Naming**: Use a composite of the higher-order component's name and the passed-in component's name as the `displayName` on the generated component. For example, the higher-order component `withFoo()`, when passed a component `Bar` should produce a component with a `displayName` of `withFoo(Bar)`.

    > Why? A component's `displayName` may be used by developer tools or in error messages, and having a value that clearly expresses this relationship helps people understand what is happening.

```jsx
// bad
export default function withFoo(WrappedComponent) {
  return function WithFoo(props) {
    return <WrappedComponent {...props} foo />;
  }
}

// good
export default function withFoo(WrappedComponent) {
  function WithFoo(props) {
    return <WrappedComponent {...props} foo />;
  }

  const wrappedComponentName = WrappedComponent.displayName
    || WrappedComponent.name
    || 'Component';

  WithFoo.displayName = `withFoo(${wrappedComponentName})`;
  return WithFoo;
}
```

- **Props Naming**: Avoid using DOM component prop names for different purposes.

    > Why? People expect props like `style` and `className` to mean one specific thing. Varying this API for a subset of your app makes the code less readable and less maintainable, and may cause bugs.

```jsx
// bad
<MyComponent style='fancy' />

// good
<MyComponent variant='fancy' />
```

## Alignment

- Follow these alignment styles for JSX syntax.

```jsx
// bad
<Foo
  superLongParam='bar'
  anotherSuperLongParam='baz'
/>

// good
<Foo superLongParam='bar'
     anotherSuperLongParam='baz' />

// if props fit in one line then keep it on the same line
<Foo bar='bar' />

// children get indented normally
<Foo
  superLongParam='bar'
  anotherSuperLongParam='baz' >
  <Quux />
</Foo>
```

## Quotes

- Always use single quotes (`'`) for all attributes.

    > Why? We use single quotes for JS and HTML, so aim for consistency.

```jsx
// bad
<Foo bar="bar" />

// good
<Foo bar='bar' />

// bad
<Foo style={{ left: "20px" }} />

// good
<Foo style={{ left: '20px' }} />
```

## Spacing

- Always include a single space in your self-closing tag. eslint: [`no-multi-spaces`](http://eslint.org/docs/rules/no-multi-spaces), [`react/jsx-space-before-closing`](https://github.com/yannickcr/eslint-plugin-react/blob/master/docs/rules/jsx-space-before-closing.md)

```jsx
// bad
<Foo/>

// very bad
<Foo                 />

// bad
<Foo
 />

// good
<Foo />
```

- Do not pad JSX curly braces with spaces.

```jsx
// bad
<Foo bar={ baz } />

// good
<Foo bar={baz} />
```

## Props

- Always use camelCase for prop names.

```jsx
// bad
<Foo
  UserName='hello'
  phone_number={12345678} />

// good
<Foo
  userName='hello'
  phoneNumber={12345678} />
```

- Omit the value of the prop when it is explicitly `true`.

```jsx
// bad
<Foo
  hidden={true}
/>

// good
<Foo
  hidden
/>
```

- Always include an `alt` prop on `<img>` tags. If the image is presentational, `alt` can be an empty string or the `<img>` must have `role='presentation'`. eslint: [`jsx-a11y/img-has-alt`](https://github.com/evcohen/eslint-plugin-jsx-a11y/blob/master/docs/rules/img-has-alt.md)

```jsx
// bad
<img src='hello.jpg' />

// good
<img src='hello.jpg' alt='Me waving hello' />

// good
<img src='hello.jpg' alt='' />

// good
<img src='hello.jpg' role='presentation' />
```

- Do not use words like "image", "photo", or "picture" in `<img>` `alt` props. eslint: [`jsx-a11y/img-redundant-alt`](https://github.com/evcohen/eslint-plugin-jsx-a11y/blob/master/docs/rules/img-redundant-alt.md)

  > Why? Screenreaders already announce `img` elements as images, so there is no need to include this information in the alt text.

```jsx
// bad
<img src='hello.jpg' alt='Picture of me waving hello' />

// good
<img src='hello.jpg' alt='Me waving hello' />
```

- Do not use `accessKey` on elements. eslint: [`jsx-a11y/no-access-key`](https://github.com/evcohen/eslint-plugin-jsx-a11y/blob/master/docs/rules/no-access-key.md)

    > Why? Inconsistencies between keyboard shortcuts and keyboard commands used by people using screenreaders and keyboards complicate accessibility.

```jsx
// bad
<div accessKey='h' />

// good
<div />
```

- Avoid using an array index as `key` prop, prefer a unique ID.

    > [Why?](https://medium.com/@robinpokorny/index-as-a-key-is-an-anti-pattern-e0349aece318))

```jsx
// bad
{todos.map((todo, index) =>
  <Todo
    {...todo}
    key={index}
  />
)}

// good
{todos.map(todo => (
  <Todo
    {...todo}
    key={todo.id}
  />
))}
```

- Always define explicit defaultProps for all non-required props.

    > Why? propTypes are a form of documentation, and providing defaultProps means the reader of your code doesn’t have to assume as much. In addition, it can mean that your code can omit certain type checks.

```jsx
// bad
function SFC({ foo, bar, children }) {
  return <div>{foo}{bar}{children}</div>;
}

SFC.propTypes = {
  foo: PropTypes.number.isRequired,
  bar: PropTypes.string,
  children: PropTypes.node,
};

// good
function SFC({ foo, bar }) {
  return <div>{foo}{bar}</div>;
}

SFC.propTypes = {
  foo: PropTypes.number.isRequired,
  bar: PropTypes.string,
  children: PropTypes.node,
};

SFC.defaultProps = {
  bar: '',
  children: null,
};
```

## Refs

- Always use ref callbacks.
  eslint: [`react/no-string-refs`](https://github.com/yannickcr/eslint-plugin-react/blob/master/docs/rules/no-string-refs.md)

```jsx
// bad
<Foo
  ref='myRef' />

// good
<Foo
  ref={(ref) => { this.myRef = ref; }} />
```

## Parentheses

- Wrap JSX tags in parentheses when they span more than one line. eslint: [`react/wrap-multilines`](https://github.com/yannickcr/eslint-plugin-react/blob/master/docs/rules/wrap-multilines.md)

```jsx
// bad
render() {
  return <MyComponent className='long body' foo='bar'>
           <MyChild />
         </MyComponent>;
}

// good
render() {
  return (
    <MyComponent className='long body' foo='bar'>
      <MyChild />
    </MyComponent>
  );
}

// good, when single line
render() {
  const body = <div>hello</div>;
  return <MyComponent>{body}</MyComponent>;
}
```

## Tags

- Always self-close tags that have no children. eslint: [`react/self-closing-comp`](https://github.com/yannickcr/eslint-plugin-react/blob/master/docs/rules/self-closing-comp.md)

```jsx
// bad
<Foo className='stuff'></Foo>

// good
<Foo className='stuff' />
```

- If your component has multi-line properties, close its tag on athe last property.

```jsx
// bad
<Foo
  bar='bar'
  baz='baz'
  />

<Foo
  bar='bar'
  baz='baz'
/>

// good
<Foo
  bar='bar'
  baz='baz' />
```

## Methods

- Use arrow functions to close over local variables.

```jsx
function ItemList(props) {
  return (
    <ul>
      {props.items.map((item, index) => (
        <Item
          key={item.key}
          onClick={() => doSomethingWith(item.name, index)}
        />
      ))}
    </ul>
  );
}
```

- Use underscore prefix for internal methods of a React component.

  > Why? Even though there is no native support for privacy in JavaScript, (everything is public), it is a convention to indicate what method is used only internally and so anything specific to React is easy to find.

```jsx
(modified to our style)
// bad
React.createClass({
  handleClickSubmit() {
    // do stuff
  }

  // other stuff
}

// good
class extends React.Component {
  _handleClickSubmit() {
    // do stuff
  },

  // other stuff
});
```

- Be sure to return a value in your `render` methods. eslint: [`react/require-render-return`](https://github.com/yannickcr/eslint-plugin-react/blob/master/docs/rules/require-render-return.md)

```jsx
// bad
render() {
  (<div />);
}

// good
render() {
  return (<div />);
}
```

## Event handlers

- Dynamically create event handlers and always define a default one, usually `onClick`.

  Functions passed to the component which contain the actions to specific events should be named with the prefix `on`, e.g. `onTapped`, `onHoverIn`.

  Internal functions that are calling such callbacks and handle other component-specific behavior should have the prefix `handle`, e.g. `handleTapped`, `handleComponentClosed`.

  In other words, ```onSomethingHappened``` for props / public api methods ```_handleSomethingHappened``` for responding to callbacks fired from ```onSomethingHappened``` callback.

```jsx
const Satellite = ({
  . . .
  onTapped,
  onHoverIn,
  onHoverOut,
  . . .  
  }) => {

// . . .

let eventHandlerProps = {};

const handleTapped = () => {
  onTapped(data);
};

if (onTapped && typeof onTapped === 'function') {
  eventHandlerProps[tappedPropName] = handleTapped;
}

if (onHoverIn && typeof onHoverIn === 'function') {
  eventHandlerProps.onMouseEnter = () => onHoverIn(data);
}

if (onHoverOut && typeof onHoverOut === 'function') {
  eventHandlerProps.onMouseLeave = () => onHoverOut(data);
}

return (
  <g
    className='qb-satellite'
    {...eventHandlerProps}>
    . . .
  </g>
);
};

Satellite.defaultProps = {
. . .
tappedPropName: 'onClick'
};
```

## Ordering

- Ordering for `class extends React.Component`
    eslint: [`react/sort-comp`](https://github.com/yannickcr/eslint-plugin-react/blob/master/docs/rules/sort-comp.md)

    1. optional `static` methods
    1. `constructor`
    1. `getChildContext`
    1. `componentWillMount`
    1. `componentDidMount`
    1. `componentWillReceiveProps`
    1. `shouldComponentUpdate`
    1. `componentWillUpdate`
    1. `componentDidUpdate`
    1. `componentWillUnmount`
    1. *clickHandlers or eventHandlers* like `onClickSubmit()` or `onChangeDescription()`
    1. *getter methods for `render`* like `getSelectReason()` or `getFooterContent()`
    1. *optional render methods* like `renderNavigation()` or `renderProfilePicture()`
    1. `render`

- How to define `propTypes`, `defaultProps`, `contextTypes`, etc...

```jsx
import React, { PropTypes } from 'react';

const propTypes = {
  id: PropTypes.number.isRequired,
  url: PropTypes.string.isRequired,
  text: PropTypes.string,
};

const defaultProps = {
  text: 'Hello World',
};

class Link extends React.Component {
  static methodsAreOk() {
    return true;
  }

  render() {
    return <a href={this.props.url} data-id={this.props.id}>{this.props.text}</a>
  }
}

Link.propTypes = propTypes;
Link.defaultProps = defaultProps;

export default Link;
```

## `isMounted`

- Do not use `isMounted`. eslint: [`react/no-is-mounted`](https://github.com/yannickcr/eslint-plugin-react/blob/master/docs/rules/no-is-mounted.md)

  > Why? [`isMounted` is an anti-pattern][anti-pattern], is not available when using ES6 classes, and is on its way to being officially deprecated.

  [anti-pattern]: https://facebook.github.io/react/blog/2015/12/16/ismounted-antipattern.html


**[⬆ back to top](#table-of-contents)**
